#!/bin/bash
#
# buildOpencv.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e17|c2>[:py3]  [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=opencv
origpkgver=v3_4_2
pkgver=${origpkgver}a
ssibuildshims_version=v1_04_13
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=opencv-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics NO_TYPE

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${basequal}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [[ "${basequal}" == e1[045] ]]
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++14"
  extra_commands="-DENABLE_CXX14=ON"
elif [[ "${basequal}" == e1[79] ]] || [[ "${basequal}" == e1[79]:py3 ]]
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++17 -Wno-register -Wno-terminate"
  extra_commands="-DENABLE_CXX17=ON"
elif [[ "${basequal}" == c[27] ]] || [[ "${basequal}" == c[27]:py3 ]]
then
  cc=clang
  cxx=clang++
  cxxflg="-std=c++17 -Wno-register -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
  extra_commands="-DENABLE_CXX17=ON"
else
  ssi_die "Qualifier $basequal not recognized."
fi

# if there was an old build, remove it
rm -rf ${pkgdir}
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
rm -rf ${blddir}
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${basequal}

setup -B ${package} ${pkgver} -q ${basequal} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

# opencv depends on python
if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup camke"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

cd ${pkgdir}
mkdir src || ssi_die "failed to create ${pkgdir}/src"
cd src
tar xf ${tardir}/${pkgtarname}.tar.gz || ssi_die "unwind of ${pkgtarname}.tar.gz failed"

patch -b -p0 < ${patchdir}/${pkgtarname}.patch || ssi_die "cannot apply patch ${pkgtarname}.patch"
patch -b -p0 < ${patchdir}/cxx17.patch || ssi_die "cannot apply patch cxx17.patch"

set -x

cd ${blddir}

##env CC=gcc CXX=g++ FC=gfortran CXXFLAGS="-DENABLE_TORCH_IMPORTER=0" cmake \
env CC=${cc} CXX=${cxx} FC=gfortran cmake \
   -DCMAKE_BUILD_TYPE=RELEASE \
   -DCMAKE_CXX_FLAGS="${cxxflg}" \
   -DBUILD_SHARED_LIBS=ON \
   ${extra_commands} \
   -DWITH_CUDA=OFF \
   -DWITH_IPP=OFF \
   -DWITH_EIGEN=OFF \
   -DWITH_JASPER=OFF \
   -DWITH_LAPACK=OFF \
   -DWITH_MATLAB=OFF \
   -DWITH_VTK=OFF \
   -DWITH_WEBP=OFF \
   -DWITH_TIFF=OFF \
   -DWITH_QT=OFF \
   -DWITH_GSTREAMER=OFF \
   -DWITH_GPHOTO2=OFF \
   -DCV_ENABLE_INTRINSICS=OFF \
   -DENABLE_CCACHE=OFF \
   -DBUILD_JAVA=OFF \
   -DBUILD_OPENEXR=ON \
   -DBUILD_opencv_python2=ON \
   -DBUILD_opencv_python3=OFF \
   -DPYTHON2_LIBRARY=${PYTHONHOME} \
   -DBUILD_TESTS=OFF \
   -DBUILD_PERF_TESTS=OFF \
   -DINSTALL_TESTS=OFF \
   -DINSTALL_PYTHON_EXAMPLES=OFF \
   -DCMAKE_INSTALL_PREFIX="${pkgdir}" \
   ${pkgdir}/src/${pkgtarname} || ssi_die "cmake failed"
make -j ${ncore} || ssi_die "ERROR: make failed."
make install || ssi_die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ] && [ ! -d ${pkgdir}/lib64 ]
then
   echo "ERROR: failed to create ${pkgdir}/lib or ${pkgdir}/lib64"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${basequal} || ssi_die "failed to declare ${package} ${pkgver}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${basequal} -z ${product_dir}:${PRODUCTS}
echo "opencv is installed at ${OPENCV_FQ_DIR}"


# fixing pkgconfig file
mv ${OPENCV_FQ_DIR}/lib64/pkgconfig/opencv.pc ${OPENCV_FQ_DIR}/lib64/pkgconfig/opencv.pc.bak
cat ${OPENCV_FQ_DIR}/lib64/pkgconfig/opencv.pc.bak | sed "s%${OPENCV_FQ_DIR}%\${pcfiledir}/../..%g" > ${OPENCV_FQ_DIR}/lib64/pkgconfig/opencv.pc

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${OPENCV_FQ_DIR} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${basequal}
fi

exit 0
